from dataclasses import dataclass
from datetime import datetime

from psycopg import sql
from psycopg.rows import class_row
from psycopg_pool import AsyncConnectionPool
from pydantic import constr


@dataclass
class Todo:
    complete: bool
    due: datetime | None
    id: int
    task: constr(strip_whitespace=True, min_length=1)  # type: ignore


async def select_todos(
    db: AsyncConnectionPool, member_id: int, complete: bool | None = None
) -> list[Todo]:
    """
    Select multiple Todo(s) from a member_id

    Return a list of Todos (can be empty list)
    """
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Todo)) as curr:

            if complete is None:
                condition = sql.SQL("member_id = {}").format(sql.Literal(member_id))
            else:
                condition = sql.SQL("member_id = {} AND complete = {}").format(
                    sql.Literal(member_id), sql.Literal(complete)
                )

            query = sql.SQL(
                """
                SELECT id, complete, due, task
                    FROM todos
                    WHERE {}
                """
            ).format(condition)

            await curr.execute(
                query,
            )

            todos = await curr.fetchall()

            return todos


async def select_todo(db: AsyncConnectionPool, id: int, member_id: int) -> Todo | None:
    """
    Find exactly one todo task that matches id and member_id
    in todos list in PostgreSQL database

    Return a Todo object or None
    """
    async with db.connection() as conn:
        async with conn.cursor(row_factory=class_row(Todo)) as curr:
            query = sql.SQL(
                """
                SELECT id, complete, due, task
                    FROM todos
                    WHERE id = (%s) AND member_id = (%s)
                """
            )

            await curr.execute(query, (id, member_id))

            member = await curr.fetchone()

            return None if member is None else member
